@@ -15,20 +15,19 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
+#include "_stdint.h"
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <errno.h>
 
@@ -36,7 +35,7 @@
 
 #include <gmodule.h>
 
-#include <glib/gi18n-lib.h>
+#include <gst/gst-i18n-plugin.h>
 
 GST_DEBUG_CATEGORY_STATIC (gstgst_dvd_read_src_debug);
 #define GST_CAT_DEFAULT (gstgst_dvd_read_src_debug)
@@ -78,7 +77,7 @@ static void gst_dvd_read_src_set_property (GObject * object, guint prop_id,
 static void gst_dvd_read_src_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static GstEvent *gst_dvd_read_src_make_clut_change_event (GstDvdReadSrc * src,
-    const guint32 * clut);
+    const guint * clut);
 static gboolean gst_dvd_read_src_get_size (GstDvdReadSrc * src, gint64 * size);
 static gboolean gst_dvd_read_src_do_seek (GstBaseSrc * src, GstSegment * s);
 static gint64 gst_dvd_read_src_convert_timecode (dvd_time_t * time);
@@ -91,13 +90,11 @@ static gint gst_dvd_read_src_get_sector_from_time (GstDvdReadSrc * src,
 
 static void gst_dvd_read_src_uri_handler_init (gpointer g_iface,
     gpointer iface_data);
-static gboolean dvdread_element_init (GstPlugin * plugin);
 
 #define gst_dvd_read_src_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstDvdReadSrc, gst_dvd_read_src, GST_TYPE_PUSH_SRC,
     G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER,
         gst_dvd_read_src_uri_handler_init));
-GST_ELEMENT_REGISTER_DEFINE_CUSTOM (dvdreadsrc, dvdread_element_init);
 
 static void
 gst_dvd_read_src_finalize (GObject * object)
@@ -118,7 +115,6 @@ gst_dvd_read_src_init (GstDvdReadSrc * src)
   src->dvd_title = NULL;
 
   src->location = g_strdup ("/dev/dvd");
-  src->first_seek = TRUE;
   src->new_seek = TRUE;
   src->new_cell = TRUE;
   src->change_cell = FALSE;
@@ -166,7 +162,8 @@ gst_dvd_read_src_class_init (GstDvdReadSrcClass * klass)
       g_param_spec_int ("angle", "angle", "angle",
           1, 999, 1, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_static_pad_template (gstelement_class, &srctemplate);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&srctemplate));
 
   gst_element_class_set_static_metadata (gstelement_class, "DVD Source",
       "Source/File/DVD",
@@ -222,8 +219,6 @@ gst_dvd_read_src_start (GstBaseSrc * basesrc)
   src->new_seek = FALSE;
   src->change_cell = TRUE;
 
-  src->first_seek = TRUE;
-
   return TRUE;
 
   /* ERRORS */
@@ -350,7 +345,6 @@ static gboolean
 gst_dvd_read_src_goto_chapter (GstDvdReadSrc * src, gint chapter)
 {
   gint i;
-  const guint8 *palette;
 
   /* make sure the chapter number is valid for this title */
   if (chapter < 0 || chapter >= src->num_chapters) {
@@ -393,11 +387,8 @@ gst_dvd_read_src_goto_chapter (GstDvdReadSrc * src, gint chapter)
   if (src->pending_clut_event)
     gst_event_unref (src->pending_clut_event);
 
-  /* Work around GCC 9 compiler warning here about taking address of packed
-   * member, which may result in an unaligned pointer access */
-  palette = (const guint8 *) src->cur_pgc->palette;
   src->pending_clut_event =
-      gst_dvd_read_src_make_clut_change_event (src, (const guint32 *) palette);
+      gst_dvd_read_src_make_clut_change_event (src, src->cur_pgc->palette);
 
   return TRUE;
 }
@@ -565,7 +556,6 @@ gst_dvd_read_src_goto_title (GstDvdReadSrc * src, gint title, gint angle)
       lang_code[0] = (u->lang_code >> 8) & 0xff;
       lang_code[1] = u->lang_code & 0xff;
       gst_structure_set (s, t, G_TYPE_STRING, lang_code, NULL);
-      g_free (t);
       t = g_strdup_printf ("subpicture-%d-stream", i);
       gst_structure_set (s, t, G_TYPE_INT, (int) sid, NULL);
       g_free (t);
@@ -600,7 +590,7 @@ gst_dvd_read_src_goto_title (GstDvdReadSrc * src, gint title, gint angle)
       for (j = 0; j < src->vts_tmapt->tmap[i].nr_of_entries; j++) {
         guint64 time;
 
-        time = (guint64) src->vts_tmapt->tmap[i].tmu * (j + 1) * GST_SECOND;
+        time = src->vts_tmapt->tmap[i].tmu * (j + 1) * GST_SECOND;
         GST_LOG_OBJECT (src, "Time: %" GST_TIME_FORMAT " VOBU "
             "Sector: 0x%08x %s", GST_TIME_ARGS (time),
             src->vts_tmapt->tmap[i].map_ent[j] & 0x7fffffff,
@@ -705,7 +695,7 @@ gst_dvd_read_src_get_time_for_sector (GstDvdReadSrc * src, guint sector)
   for (i = 0; i < src->vts_tmapt->nr_of_tmaps; ++i) {
     for (j = 0; j < src->vts_tmapt->tmap[i].nr_of_entries; ++j) {
       if ((src->vts_tmapt->tmap[i].map_ent[j] & 0x7fffffff) == sector)
-        return (guint64) src->vts_tmapt->tmap[i].tmu * (j + 1) * GST_SECOND;
+        return src->vts_tmapt->tmap[i].tmu * (j + 1) * GST_SECOND;
     }
   }
 
@@ -724,13 +714,11 @@ gst_dvd_read_src_get_sector_from_time (GstDvdReadSrc * src, GstClockTime ts)
   if (src->vts_tmapt == NULL || src->vts_tmapt->nr_of_tmaps < src->ttn)
     return -1;
 
-  sector = src->vts_tmapt->tmap[src->ttn - 1].map_ent[0] & 0x7fffffff;
-
+  sector = 0;
   for (j = 0; j < src->vts_tmapt->tmap[src->ttn - 1].nr_of_entries; ++j) {
     GstClockTime entry_time;
 
-    entry_time =
-        (guint64) src->vts_tmapt->tmap[src->ttn - 1].tmu * (j + 1) * GST_SECOND;
+    entry_time = src->vts_tmapt->tmap[src->ttn - 1].tmu * (j + 1) * GST_SECOND;
     if (entry_time <= ts) {
       sector = src->vts_tmapt->tmap[src->ttn - 1].map_ent[j] & 0x7fffffff;
     }
@@ -1031,7 +1019,7 @@ gst_dvd_read_src_set_property (GObject * object, guint prop_id,
       if (started) {
         g_warning ("%s: property '%s' needs to be set before the device is "
             "opened", GST_ELEMENT_NAME (src), pspec->name);
-        break;
+        break;;
       }
 
       g_free (src->location);
@@ -1039,7 +1027,7 @@ gst_dvd_read_src_set_property (GObject * object, guint prop_id,
       if (g_value_get_string (value) == NULL) {
         src->location = g_strdup ("/dev/dvd");
       } else {
-        src->location = g_value_dup_string (value);
+        src->location = g_strdup (g_value_get_string (value));
       }
       break;
     }
@@ -1250,13 +1238,6 @@ gst_dvd_read_src_do_seek (GstBaseSrc * basesrc, GstSegment * s)
   GST_DEBUG_OBJECT (src, "Seeking to %s: %12" G_GINT64_FORMAT,
       gst_format_get_name (s->format), s->position);
 
-  /* Ignore the first seek to 0, as it breaks starting playback
-   * from another chapter by seeking back to sector 0 */
-  if (src->first_seek && s->format == GST_FORMAT_BYTES && s->start == 0) {
-    src->first_seek = FALSE;
-    return TRUE;
-  }
-
   if (s->format == sector_format || s->format == GST_FORMAT_BYTES
       || s->format == GST_FORMAT_TIME) {
     guint old;
@@ -1354,7 +1335,7 @@ gst_dvd_read_src_src_event (GstBaseSrc * basesrc, GstEvent * event)
 
 static GstEvent *
 gst_dvd_read_src_make_clut_change_event (GstDvdReadSrc * src,
-    const guint32 * clut)
+    const guint * clut)
 {
   GstStructure *structure;
   gchar name[16];
@@ -1370,7 +1351,7 @@ gst_dvd_read_src_make_clut_change_event (GstDvdReadSrc * src,
   }
 
   /* Create the DVD event and put the structure into it. */
-  return gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM_STICKY, structure);
+  return gst_event_new_custom (GST_EVENT_CUSTOM_DOWNSTREAM, structure);
 }
 
 static gint64
@@ -1398,7 +1379,7 @@ static gboolean
 gst_dvd_read_src_do_duration_query (GstDvdReadSrc * src, GstQuery * query)
 {
   GstFormat format;
-  gint64 val = 0;
+  gint64 val;
 
   gst_query_parse_duration (query, &format, NULL);
 
@@ -1791,7 +1772,7 @@ gst_dvd_read_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
 }
 
 static gboolean
-dvdread_element_init (GstPlugin * plugin)
+plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (gstgst_dvd_read_src_debug, "dvdreadsrc", 0,
       "DVD reader element based on dvdreadsrc");
@@ -1802,14 +1783,13 @@ dvdread_element_init (GstPlugin * plugin)
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif /* ENABLE_NLS */
-  return gst_element_register (plugin, "dvdreadsrc", GST_RANK_NONE,
-      GST_TYPE_DVD_READ_SRC);
-}
 
-static gboolean
-plugin_init (GstPlugin * plugin)
-{
-  return GST_ELEMENT_REGISTER (dvdreadsrc, plugin);
+  if (!gst_element_register (plugin, "dvdreadsrc", GST_RANK_NONE,
+          GST_TYPE_DVD_READ_SRC)) {
+    return FALSE;
+  }
+
+  return TRUE;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
