@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /* FIXME:
@@ -29,8 +29,6 @@
 #include <gst/gstinfo.h>
 #include <string.h>
 
-#define GST_ASF_PAYLOAD_KF_COMPLETE(stream, payload) (stream->is_video && payload->keyframe && payload->buf_filled >= payload->mo_size)
-
 /* we are unlikely to deal with lengths > 2GB here any time soon, so just
  * return a signed int and use that for error reporting */
 static inline gint
@@ -91,72 +89,31 @@ asf_packet_create_payload_buffer (AsfPacket * packet, const guint8 ** p_data,
 }
 
 static AsfPayload *
-asf_payload_search_payloads_queue (AsfPayload * payload, GArray * payload_list)
+asf_payload_find_previous_fragment (AsfPayload * payload, AsfStream * stream)
 {
-  AsfPayload *ret = NULL;
-  gint idx;
-  for (idx = payload_list->len - 1; idx >= 0; idx--) {
-    ret = &g_array_index (payload_list, AsfPayload, idx);
+  AsfPayload *ret;
 
-    if (G_UNLIKELY (ret->mo_size == payload->mo_size &&
-            ret->mo_number == payload->mo_number)) {
-      return ret;
-    }
+  if (G_UNLIKELY (stream->payloads->len == 0)) {
+    GST_DEBUG ("No previous fragments to merge with for stream %u", stream->id);
+    return NULL;
   }
-  return NULL;
-}
 
-static AsfPayload *
-asf_payload_find_previous_fragment (GstASFDemux * demux, AsfPayload * payload,
-    AsfStream * stream)
-{
-  AsfPayload *ret = NULL;
+  ret =
+      &g_array_index (stream->payloads, AsfPayload, stream->payloads->len - 1);
 
-  if (GST_ASF_DEMUX_IS_REVERSE_PLAYBACK (demux->segment)) {
-
-    /* Search in queued payloads list */
-    ret = asf_payload_search_payloads_queue (payload, stream->payloads);
-    if (ret) {
-      GST_DEBUG
-          ("previous fragments found in payloads queue for reverse playback : object ID %d",
-          ret->mo_number);
-      return ret;
-    }
-
-    /* Search in payloads 'to be queued' list */
-    ret = asf_payload_search_payloads_queue (payload, stream->payloads_rev);
-    if (ret) {
-      GST_DEBUG
-          ("previous fragments found in temp payload queue for reverse playback : object ID %d",
-          ret->mo_number);
-      return ret;
-    }
-  } else {
-    if (G_UNLIKELY (stream->payloads->len == 0)) {
-      GST_DEBUG ("No previous fragments to merge with for stream %u",
-          stream->id);
+  if (G_UNLIKELY (ret->mo_size != payload->mo_size ||
+          ret->mo_number != payload->mo_number || ret->mo_offset != 0)) {
+    if (payload->mo_size != 0) {
+      GST_WARNING ("Previous fragment does not match continued fragment");
       return NULL;
-    }
-
-    ret =
-        &g_array_index (stream->payloads, AsfPayload,
-        stream->payloads->len - 1);
-
-    if (G_UNLIKELY (ret->mo_size != payload->mo_size ||
-            ret->mo_number != payload->mo_number || ret->mo_offset != 0)) {
-      if (payload->mo_size != 0) {
-        GST_WARNING ("Previous fragment does not match continued fragment");
-        return NULL;
-      } else {
-        /* Warn about this case, but accept it anyway: files in the wild sometimes
-         * have continued packets where the subsequent fragments say that they're
-         * zero-sized. */
-        GST_WARNING ("Previous fragment found, but current fragment has "
-            "zero size, accepting anyway");
-      }
+    } else {
+      /* Warn about this case, but accept it anyway: files in the wild sometimes
+       * have continued packets where the subsequent fragments say that they're
+       * zero-sized. */
+      GST_WARNING ("Previous fragment found, but current fragment has "
+          "zero size, accepting anyway");
     }
   }
-
 #if 0
   if (this_fragment->mo_offset + this_payload_len > first_fragment->mo_size) {
     GST_WARNING ("Merged fragments would be bigger than the media object");
@@ -171,22 +128,33 @@ asf_payload_find_previous_fragment (GstASFDemux * demux, AsfPayload * payload,
  * payload doesn't have a duration, maybe we can calculate a duration for it
  * (if the previous timestamp is smaller etc. etc.) */
 static void
-gst_asf_payload_queue_for_stream_forward (GstASFDemux * demux,
-    AsfPayload * payload, AsfStream * stream)
+gst_asf_payload_queue_for_stream (GstASFDemux * demux, AsfPayload * payload,
+    AsfStream * stream)
 {
   GST_DEBUG_OBJECT (demux, "Got payload for stream %d ts:%" GST_TIME_FORMAT,
       stream->id, GST_TIME_ARGS (payload->ts));
 
-  /* make timestamps start from 0; first_ts will be determined during activation (once we have enough data),
-     which will also update ts of all packets queued before we knew first_ts;  */
-  if (G_LIKELY (GST_CLOCK_TIME_IS_VALID (demux->first_ts)
-          && GST_CLOCK_TIME_IS_VALID (payload->ts))) {
-    if (payload->ts > demux->first_ts)
-      payload->ts -= demux->first_ts;
-    else
-      payload->ts = 0;
+  /* Before preroll ts might be invalid and set to 0 */
+  if (G_UNLIKELY (payload->ts == 0 && demux->preroll)) {
+    payload->ts = GST_CLOCK_TIME_NONE;
   }
 
+  /* remember the first timestamp in the stream */
+  if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (demux->first_ts) &&
+          GST_CLOCK_TIME_IS_VALID (payload->ts))) {
+    GST_DEBUG_OBJECT (demux, "first ts: %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (payload->ts));
+    demux->first_ts = payload->ts;
+  }
+
+  /* make timestamps start from 0 */
+  if (G_LIKELY (GST_CLOCK_TIME_IS_VALID (demux->first_ts) &&
+          GST_CLOCK_TIME_IS_VALID (payload->ts) &&
+          demux->first_ts < payload->ts))
+    payload->ts -= demux->first_ts;
+  else
+    payload->ts = 0;
+
   /* remove any incomplete payloads that will never be completed */
   while (stream->payloads->len > 0) {
     AsfPayload *prev;
@@ -233,82 +201,51 @@ gst_asf_payload_queue_for_stream_forward (GstASFDemux * demux,
     GST_BUFFER_FLAG_SET (payload->buf, GST_BUFFER_FLAG_DISCONT);
   }
 
+  /* remember the first queued timestamp for the segment */
+  if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (demux->segment_ts) &&
+          GST_CLOCK_TIME_IS_VALID (payload->ts))) {
+    GST_DEBUG_OBJECT (demux, "segment ts: %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (payload->ts));
+    demux->segment_ts = payload->ts;
+    /* always note, but only determines segment when streaming */
+    if (demux->streaming)
+      gst_segment_do_seek (&demux->segment, demux->in_segment.rate,
+          GST_FORMAT_TIME, (GstSeekFlags) demux->segment.flags,
+          GST_SEEK_TYPE_SET, demux->segment_ts, GST_SEEK_TYPE_NONE, 0, NULL);
+  }
+
   g_array_append_vals (stream->payloads, payload, 1);
 }
 
-static void
-gst_asf_payload_queue_for_stream_reverse (GstASFDemux * demux,
-    AsfPayload * payload, AsfStream * stream)
-{
-  GST_DEBUG_OBJECT (demux, "Got payload for stream %d ts:%" GST_TIME_FORMAT,
-      stream->id, GST_TIME_ARGS (payload->ts));
-
-  if (demux->multiple_payloads) {
-    /* store the payload in temporary buffer, until we parse all payloads in this packet */
-    g_array_append_vals (stream->payloads_rev, payload, 1);
-  } else {
-    if (G_LIKELY (GST_CLOCK_TIME_IS_VALID (payload->ts))) {
-      g_array_append_vals (stream->payloads, payload, 1);
-      if (GST_ASF_PAYLOAD_KF_COMPLETE (stream, payload)) {
-        stream->kf_pos = stream->payloads->len - 1;
-      }
-    } else {
-      gst_buffer_unref (payload->buf);
-    }
-  }
-}
-
-
-static void
-gst_asf_payload_queue_for_stream (GstASFDemux * demux, AsfPayload * payload,
-    AsfStream * stream)
-{
-  GST_DEBUG_OBJECT (demux, "Got payload for stream %d ts:%" GST_TIME_FORMAT,
-      stream->id, GST_TIME_ARGS (payload->ts));
-
-  if (GST_ASF_DEMUX_IS_REVERSE_PLAYBACK (demux->segment)) {
-    gst_asf_payload_queue_for_stream_reverse (demux, payload, stream);
-  } else {
-    gst_asf_payload_queue_for_stream_forward (demux, payload, stream);
-  }
-
-}
-
 static void
 asf_payload_parse_replicated_data_extensions (AsfStream * stream,
     AsfPayload * payload)
 {
   AsfPayloadExtension *ext;
   guint off;
-  guint16 ext_len;
 
   if (!stream->ext_props.valid || stream->ext_props.payload_extensions == NULL)
     return;
 
   off = 8;
   for (ext = stream->ext_props.payload_extensions; ext->len > 0; ++ext) {
-    ext_len = ext->len;
-    if (ext_len == 0xFFFF) {    /* extension length is determined by first two bytes in replicated data */
-      ext_len = GST_READ_UINT16_LE (payload->rep_data + off);
-      off += 2;
-    }
-    if (G_UNLIKELY (off + ext_len > payload->rep_data_len)) {
+    if (G_UNLIKELY (off + ext->len > payload->rep_data_len)) {
       GST_WARNING ("not enough replicated data for defined extensions");
       return;
     }
     switch (ext->id) {
       case ASF_PAYLOAD_EXTENSION_DURATION:
-        if (G_LIKELY (ext_len == 2)) {
+        if (G_LIKELY (ext->len == 2)) {
           guint16 tdur = GST_READ_UINT16_LE (payload->rep_data + off);
           /* packet durations of 1ms are mostly invalid */
           if (tdur != 1)
             payload->duration = tdur * GST_MSECOND;
         } else {
-          GST_WARNING ("unexpected DURATION extensions len %u", ext_len);
+          GST_WARNING ("unexpected DURATION extensions len %u", ext->len);
         }
         break;
       case ASF_PAYLOAD_EXTENSION_SYSTEM_CONTENT:
-        if (G_LIKELY (ext_len == 1)) {
+        if (G_LIKELY (ext->len == 1)) {
           guint8 data = payload->rep_data[off];
 
           payload->interlaced = data & 0x1;
@@ -317,34 +254,24 @@ asf_payload_parse_replicated_data_extensions (AsfStream * stream,
           GST_DEBUG ("SYSTEM_CONTENT: interlaced:%d, rff:%d, tff:%d",
               payload->interlaced, payload->rff, payload->tff);
         } else {
-          GST_WARNING ("unexpected SYSTEM_CONTE extensions len %u", ext_len);
+          GST_WARNING ("unexpected SYSTEM_CONTE extensions len %u", ext->len);
         }
         break;
       case ASF_PAYLOAD_EXTENSION_SYSTEM_PIXEL_ASPECT_RATIO:
-        if (G_LIKELY (ext_len == 2)) {
+        if (G_LIKELY (ext->len == 2)) {
           payload->par_x = payload->rep_data[off];
           payload->par_y = payload->rep_data[off + 1];
           GST_DEBUG ("PAR %d / %d", payload->par_x, payload->par_y);
         } else {
           GST_WARNING ("unexpected SYSTEM_PIXEL_ASPECT_RATIO extensions len %u",
-              ext_len);
+              ext->len);
         }
         break;
-      case ASF_PAYLOAD_EXTENSION_TIMING:
-      {
-        /* dvr-ms timing - this will override packet timestamp */
-        guint64 time = GST_READ_UINT64_LE (payload->rep_data + off + 8);
-        if (time != 0xFFFFFFFFFFFFFFFF)
-          payload->ts = time * 100;
-        else
-          payload->ts = GST_CLOCK_TIME_NONE;
-      }
-        break;
       default:
-        GST_LOG ("UNKNOWN PAYLOAD EXTENSION!");
+        GST_WARNING ("UNKNOWN PAYLOAD EXTENSION !");
         break;
     }
-    off += ext_len;
+    off += ext->len;
   }
 }
 
@@ -424,10 +351,8 @@ gst_asf_demux_parse_payload (GstASFDemux * demux, AsfPacket * packet,
   stream = gst_asf_demux_get_stream (demux, stream_num);
 
   if (G_UNLIKELY (stream == NULL)) {
-    if (gst_asf_demux_is_unknown_stream (demux, stream_num)) {
-      GST_WARNING_OBJECT (demux, "Payload for unknown stream %u, skipping",
-          stream_num);
-    }
+    GST_WARNING_OBJECT (demux, "Payload for unknown stream %u, skipping",
+        stream_num);
     if (*p_size < payload_len) {
       *p_data += *p_size;
       *p_size = 0;
@@ -438,9 +363,6 @@ gst_asf_demux_parse_payload (GstASFDemux * demux, AsfPacket * packet,
     return TRUE;
   }
 
-  if (!stream->is_video)
-    stream->kf_pos = 0;
-
   if (G_UNLIKELY (!is_compressed)) {
     GST_LOG_OBJECT (demux, "replicated data length: %u", payload.rep_data_len);
 
@@ -458,8 +380,6 @@ gst_asf_demux_parse_payload (GstASFDemux * demux, AsfPacket * packet,
           GST_TIME_ARGS (payload.ts));
       GST_LOG_OBJECT (demux, "media object dur    : %" GST_TIME_FORMAT,
           GST_TIME_ARGS (payload.duration));
-    } else if (payload.rep_data_len == 0) {
-      payload.mo_size = 0;
     } else if (payload.rep_data_len != 0) {
       GST_WARNING_OBJECT (demux, "invalid replicated data length, very bad");
       *p_data += payload_len;
@@ -480,38 +400,6 @@ gst_asf_demux_parse_payload (GstASFDemux * demux, AsfPacket * packet,
           payload_len);
       payload.buf_filled = payload_len;
       gst_asf_payload_queue_for_stream (demux, &payload, stream);
-    } else if (GST_ASF_DEMUX_IS_REVERSE_PLAYBACK (demux->segment)) {
-      /* Handle fragmented payloads for reverse playback */
-      AsfPayload *prev;
-      const guint8 *payload_data = *p_data;
-      prev = asf_payload_find_previous_fragment (demux, &payload, stream);
-
-      if (prev) {
-        gint idx;
-        AsfPayload *p;
-        gst_buffer_fill (prev->buf, payload.mo_offset,
-            payload_data, payload_len);
-        prev->buf_filled += payload_len;
-        if (payload.keyframe && payload.mo_offset == 0) {
-          stream->reverse_kf_ready = TRUE;
-
-          for (idx = stream->payloads->len - 1; idx >= 0; idx--) {
-            p = &g_array_index (stream->payloads, AsfPayload, idx);
-            if (p->mo_number == payload.mo_number) {
-              /* Mark position of KF for reverse play */
-              stream->kf_pos = idx;
-            }
-          }
-        }
-      } else {
-        payload.buf = gst_buffer_new_allocate (NULL, payload.mo_size, NULL);    /* can we use (mo_size - offset) for size? */
-        gst_buffer_fill (payload.buf, payload.mo_offset,
-            payload_data, payload_len);
-        payload.buf_filled = payload.mo_size - (payload.mo_offset);
-        gst_asf_payload_queue_for_stream (demux, &payload, stream);
-      }
-      *p_data += payload_len;
-      *p_size -= payload_len;
     } else {
       const guint8 *payload_data = *p_data;
 
@@ -524,12 +412,10 @@ gst_asf_demux_parse_payload (GstASFDemux * demux, AsfPacket * packet,
       if (payload.mo_offset != 0) {
         AsfPayload *prev;
 
-        if ((prev =
-                asf_payload_find_previous_fragment (demux, &payload, stream))) {
-          if (prev->buf == NULL || (payload.mo_size > 0
-                  && payload.mo_size != prev->mo_size)
-              || payload.mo_offset >= gst_buffer_get_size (prev->buf)
-              || payload.mo_offset + payload_len >
+        if ((prev = asf_payload_find_previous_fragment (&payload, stream))) {
+          if (prev->buf == NULL || payload.mo_size != prev->mo_size ||
+              payload.mo_offset >= gst_buffer_get_size (prev->buf) ||
+              payload.mo_offset + payload_len >
               gst_buffer_get_size (prev->buf)) {
             GST_WARNING_OBJECT (demux, "Offset doesn't match previous data?!");
           } else {
@@ -704,20 +590,9 @@ gst_asf_demux_parse_packet (GstASFDemux * demux, GstBuffer * buf)
   GST_LOG_OBJECT (demux, "padding          : %u", packet.padding);
   GST_LOG_OBJECT (demux, "send time        : %" GST_TIME_FORMAT,
       GST_TIME_ARGS (packet.send_time));
-
   GST_LOG_OBJECT (demux, "duration         : %" GST_TIME_FORMAT,
       GST_TIME_ARGS (packet.duration));
 
-  if (GST_ASF_DEMUX_IS_REVERSE_PLAYBACK (demux->segment)
-      && demux->seek_to_cur_pos == TRUE) {
-    /* For reverse playback, initially parse packets forward until we reach packet with 'seek' timestamp */
-    if (packet.send_time - demux->preroll > demux->segment.stop) {
-      demux->seek_to_cur_pos = FALSE;
-    }
-    ret = GST_ASF_DEMUX_PARSE_PACKET_ERROR_NONE;
-    goto done;
-  }
-
   if (G_UNLIKELY (packet.padding == (guint) - 1 || size < packet.padding)) {
     GST_WARNING_OBJECT (demux, "No padding, or padding bigger than buffer");
     ret = GST_ASF_DEMUX_PARSE_PACKET_ERROR_RECOVERABLE;
@@ -745,7 +620,6 @@ gst_asf_demux_parse_packet (GstASFDemux * demux, GstBuffer * buf)
 
   if (has_multiple_payloads) {
     guint i, num, lentype;
-    demux->multiple_payloads = TRUE;
 
     if (G_UNLIKELY (size < 1)) {
       GST_WARNING_OBJECT (demux, "No room more in buffer");
@@ -772,29 +646,8 @@ gst_asf_demux_parse_packet (GstASFDemux * demux, GstBuffer * buf)
         break;
       }
     }
-
-    if (GST_ASF_DEMUX_IS_REVERSE_PLAYBACK (demux->segment)) {
-      /* In reverse playback, we parsed the packet (with multiple payloads) and stored the payloads in temporary queue.
-         Now, add them to the stream's payload queue */
-      for (i = 0; i < demux->num_streams; i++) {
-        AsfStream *s = &demux->stream[i];
-        while (s->payloads_rev->len > 0) {
-          AsfPayload *p;
-          p = &g_array_index (s->payloads_rev, AsfPayload,
-              s->payloads_rev->len - 1);
-          g_array_append_vals (s->payloads, p, 1);
-          if (GST_ASF_PAYLOAD_KF_COMPLETE (s, p)) {
-            /* Mark position of KF for reverse play */
-            s->kf_pos = s->payloads->len - 1;
-          }
-          g_array_remove_index (s->payloads_rev, (s->payloads_rev->len - 1));
-        }
-      }
-    }
-
   } else {
     GST_LOG_OBJECT (demux, "Parsing single payload");
-    demux->multiple_payloads = FALSE;
     if (G_UNLIKELY (!gst_asf_demux_parse_payload (demux, &packet, -1, &data,
                 &size))) {
       GST_WARNING_OBJECT (demux, "Failed to parse payload");
