@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,16 +31,25 @@
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  gboolean ret = FALSE;
+  if (!gst_rmdemux_plugin_init (plugin))
+    return FALSE;
 
-  ret |= GST_ELEMENT_REGISTER (rmdemux, plugin);
-  ret |= GST_ELEMENT_REGISTER (rademux, plugin);
-  ret |= GST_ELEMENT_REGISTER (rdtdepay, plugin);
-  ret |= GST_ELEMENT_REGISTER (rdtmanager, plugin);
-  ret |= GST_ELEMENT_REGISTER (rtspreal, plugin);
-  ret |= GST_ELEMENT_REGISTER (pnmsrc, plugin);
+  if (!gst_rademux_plugin_init (plugin))
+    return FALSE;
 
-  return ret;
+  if (!gst_rdt_depay_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rdt_manager_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rtsp_real_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_pnm_src_plugin_init (plugin))
+    return FALSE;
+
+  return TRUE;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
